import React, { Component } from 'react';
import Display from './display/Index';
import Keypad from './keypad/Index';
import Action from './action/Index';

class InitialView extends Component {
  constructor({ display }) {
    super();
    this.state = {
      display: display
    };
    this.onDigitPressed = this.onDigitPressed.bind(this);
    this.onBackPressed = this.onBackPressed.bind(this);
    this.onCallPressed = this.onCallPressed.bind(this);
    this.onPaste = this.onPaste.bind(this);

    document.addEventListener('paste', this.onPaste);
  }

  onPaste(event) {
    let paste = (event.clipboardData || window.clipboardData).getData('text') || '';

    this.setState({
      display: paste.replace(/[^\d]+/g, '')
    });

  }

  componentWillUnmount() {
    document.removeEventListener('paste', this.onPaste);
  }

  onDigitPressed(value) {
    const currentDisplay = this.state.display;

    this.setState({
      display: currentDisplay + value
    });
  }

  onBackPressed() {
    const currentDisplay = this.state.display;

    this.setState({
      display: currentDisplay.slice(0, -1)
    });
  }

  onCallPressed() {
    const { onCallPressed } = this.props;

    if (onCallPressed) {
      onCallPressed(this.state.display);
    }
  }

  render() {
    const { style } = this.props;

    return (
      <div className='initial-view' style={style}>
        <Display
          value={this.state.display}
        />
        <Keypad onDigitPressed={this.onDigitPressed} />
        <Action
          onBackPressed={this.onBackPressed}
          onCallPressed={this.onCallPressed}
        />
      </div >
    );
  }
}

export default InitialView;
